import torch
from torch.utils.data import DataLoader, RandomSampler, SequentialSampler, sampler
from torch.utils.data.distributed import DistributedSampler

from .imb_dataset import *
from .ssl_dataset import *
from .lnl_datasets import *



def get_ssl_dataloader(args):
    if args.local_rank not in [-1, 0]:
        torch.distributed.barrier()
    labeled_dataset, unlabeled_dataset, test_dataset = DATASET_GETTERS[args.dataset](args, '../database/' + args.dataset.upper())
    if args.local_rank == 0:
        torch.distributed.barrier()
    
    train_sampler = RandomSampler if args.local_rank == -1 else DistributedSampler

    labeled_trainloader = DataLoader(
        labeled_dataset,
        sampler=train_sampler(labeled_dataset),
        batch_size=args.batch_size,
        num_workers=args.num_workers,
        drop_last=True
    )
    unlabeled_trainloader = DataLoader(
        unlabeled_dataset,
        sampler=train_sampler(unlabeled_dataset),
        batch_size=args.batch_size * args.mu,
        num_workers=args.num_workers,
        drop_last=True
    )
    test_loader = DataLoader(
        test_dataset,
        sampler=SequentialSampler(test_dataset),
        batch_size=args.batch_size,
        num_workers=args.num_workers
    )
    if args.local_rank not in [-1, 0]:
        torch.distributed.barrier()
    
    return labeled_trainloader, unlabeled_trainloader, test_loader



def get_lnl_dataloader(args):
    if args.local_rank not in [-1, 0]:
        torch.distributed.barrier()
    if args.dataset in ['MNIST', 'CIFAR10', 'CIFAR100']:
        data_loader = DatasetGenerator(
            batch_size=args.batch_size,
            data_path=args.root,
            seed=args.seed,
            num_workers=args.num_workers,
            is_asym=args.noise_type=='asymmetric',
            dataset = args.dataset,
            noise_rate=args.noise_rate
        )
    elif args.dataset == 'Clothing1M':
        data_loader = Clothing1MDatasetLoader(
            data_path=args.root,
            batch_size=args.batch_size,
            num_workers=args.num_workers,
        )
    elif args.dataset == 'WEBVISION':
        data_loader = WebVisionDatasetLoader(
            train_batch_size=args.batch_size,
            eval_batch_size=args.batch_size,
            train_data_path=os.path.join(args.root, 'WebVision'),
            valid_data_path=os.path.join(args.root, 'WebVision'),
            num_of_workers=args.num_workers
        )
    else:
        raise NotImplementedError
    data_loader = data_loader.getDataLoader()
    return data_loader