import torch
import numpy as np
import torch.nn as nn
import torch.nn.functional as F


class BalancedSoftmaxLoss(nn.Module):
    def __init__(self, freq, reduction='mean'):
        super(BalancedSoftmaxLoss, self).__init__()
        self.sample_per_class = torch.tensor(freq)/ sum(freq)
        self.reduction = reduction

    def forward(self, logits, labels):
        spc = self.sample_per_class.type_as(logits)
        spc = spc.unsqueeze(0).expand(logits.shape[0], -1)
        logits = logits + spc.log()
        return F.cross_entropy(logits, labels, reduction=self.reduction)