import torch
import torch.nn as nn
import torch.nn.functional as F

class NegSampleLogitLoss(nn.Module):
    def __init__(self, weight=None, scale=1.0):
        super(NegSampleLogitLoss, self).__init__()
        self.weight = weight
        self.scale = scale

    def forward(self, logits, labels):
        label_one_hot = F.one_hot(labels, logits.size()[1]).float().to(logits.device)
        l1 = torch.sum(logits * label_one_hot, dim=1)
        loss = - l1
        if self.weight is not None:
            weight = self.weight.gather(0, labels.view(-1))
            loss = loss * weight
        return loss.mean() * self.scale
