import os
import numpy as np
import torch
import logging

from .resnet_cifar import cifar_resnet32
from .wideresnet import wideresnet
from .lnl_resnet import resnet18, resnet34
from .resnet import resnet50, resnet101, resnet152
from .ema import *
from .cnn import *
import sys
sys.path.append("../")
from prototypes.generate_prototype import generate_weight

logger = logging.getLogger(__name__)

def build_model(args, need_linear=True, mode='linear'):
    if mode.startswith('f'):
        path = './prototypes/weight1' if args.last_relu else './prototypes/weight0'
        if not os.path.exists(path + '_%dx%d.npy' % (args.num_classes, args.num_feats)):
            logger.info('==> Generate prototypes %dx%d!' % (args.num_classes, args.num_feats))
            generate_weight(args.num_classes, args.num_feats, args.last_relu)
        weight = torch.Tensor(np.load(path + '_%dx%d.npy' % (args.num_classes, args.num_feats))).to(args.device)
    else:
        weight = None
    if args.arch == 'wideresnet':
        return wideresnet(num_classes=args.num_classes, depth=args.model_depth, widen_factor=args.model_width, dropout=args.dropout, need_linear=need_linear, mode=mode, weight=weight, last_relu=args.last_relu)
    elif args.arch == 'cnn':
        return CNN(dataset=args.dataset, mode=args.mode, weight=weight, last_relu=args.last_relu)
    elif args.arch.startswith('resnet'):
        return eval(args.arch)(num_classes=args.num_classes, mode=args.mode, weight=weight, last_relu=args.last_relu if 'last_relu' in args else False)


def create_model(config, weight=None, mode='linear', two_stage=False):
    if config.dataset == 'cifar10' or config.dataset == 'cifar100':
        if two_stage:
            # model = cifar_resnet32(last_relu=config.last_relu, num_feats=config.num_feats, need_linear=False)
            model = eval(config.arch)(last_relu=config.last_relu, num_feats=config.num_feats, need_linear=config.need_linear)
            classifier = Classifier(config.num_feats, config.num_classes, mode=mode, weight=weight, need_pre=config.need_pre)
            return model, classifier, None

        else:
            model = eval(config.arch)(num_classes=config.num_classes, num_feats=config.num_feats, mode=mode, last_relu=config.last_relu, all_relu=config.all_relu, weight=weight)
            return model
    elif config.dataset == 'imagenet':
        if two_stage:
            model = eval(config.arch)(last_relu=config.last_relu, need_linear=config.need_linear, all_relu=config.all_relu)
            classifier = Classifier(config.num_feats, config.num_classes, mode=mode, weight=weight, need_pre=config.need_pre)
            return model, classifier, None
        else:
            model = eval(config.arch)(num_classes=config.num_classes, mode=mode, last_relu=config.last_relu, all_relu=config.all_relu, weight=weight, need_pre=config.need_pre)
            return model
    else:
        raise NotImplementedError