import torch
import torch.nn as nn
import torch.nn.functional as F
import torch.nn.init as init
import numpy as np
import sys
sys.path.append("../")
from losses import *

def generate_weight(n_classes, n_hiddens, use_relu=False):

    n_classes = n_classes
    n_hiddens = n_hiddens
    n_samples = n_classes * 1
    epochs = 100000

    n_per_class = n_samples // n_classes

    labels = []
    for i in range(n_classes):
        labels += [i] * n_per_class
    np.random.shuffle(labels)
    labels = torch.LongTensor(labels).cuda()

    def get_margin(weight):
        tmp = F.normalize(weight, dim=1)
        sim = torch.matmul(tmp, tmp.transpose(1, 0)) - 2 * torch.eye(tmp.size(0), device=weight.device)
        sim = torch.clamp(sim, -1+1e-7, 1-1e-7)
        return torch.acos(torch.max(sim)).item() / np.pi * 180

    def evaluate(out, labels):
        probs = F.softmax(out, dim=1)
        pred = torch.argmax(probs, 1)
        total = labels.size(0)
        correct = (pred==labels).sum().item()

        acc = float(correct) / float(total)
        return acc

    Z = torch.randn(n_samples, n_hiddens).cuda()
    Z.requires_grad = True
    W = torch.randn(n_classes, n_hiddens).cuda()
    W.requires_grad = True
    nn.init.kaiming_normal_(W)

    optimizer = torch.optim.SGD([Z, W], lr=0.1, momentum=0.9, weight_decay=1e-4)
    scheduler = torch.optim.lr_scheduler.CosineAnnealingLR(optimizer, T_max=20000, eta_min=0)

    criterion = NormFaceLoss(temp=5)
    sample_margin = SampleMarginLoss()
    score = 0
    best_weight = None
    for i in range(epochs):
        if use_relu:
            z = F.relu(Z)
        else:
            z = Z
        w = W
        L2_z = F.normalize(z, dim=1)
        L2_w = F.normalize(w, dim=1)
        out = F.linear(L2_z, L2_w)
        loss = criterion(out, labels)
        sm = - sample_margin(out, labels).item()
        optimizer.zero_grad()
        loss.backward()
        optimizer.step()
        scheduler.step()
        margin = get_margin(W)
        if score < margin * sm:
            best_weight = W.cpu().detach()
            score = margin * sm

        if i % 500 == 0:
            acc = evaluate(out, labels)
            print('Iter {}: loss={:.4f}, acc={:.4f}, SaMargin={:.4f}, ClsMargin={:.4f}'.format(i, loss.item(), acc, sm, margin))


    weight = best_weight
    weight = F.normalize(weight, dim=1)
    weight = weight.numpy()
    path = './prototypes/weight1_' if use_relu else './prototypes/weight0_'
    np.save(path + str(n_classes) + 'x' + str(n_hiddens) + '.npy', weight)


if __name__ == '__main__':
    generate_weight(50, 512 * 4, use_relu=True)